@@ -1,22 +1,22 @@
 /*
-  This file is part of GNUnet
-  Copyright (C) 2016, 2017 GNUnet e.V.
+   This file is part of GNUnet
+   Copyright (C) 2016, 2017 GNUnet e.V.
 
-  GNUnet is free software: you can redistribute it and/or modify it
-  under the terms of the GNU Affero General Public License as published
-  by the Free Software Foundation, either version 3 of the License,
-  or (at your option) any later version.
+   GNUnet is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Affero General Public License as published
+   by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
 
-  GNUnet is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Affero General Public License for more details.
- 
-  You should have received a copy of the GNU Affero General Public License
-  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   GNUnet is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Affero General Public License for more details.
+
+   You should have received a copy of the GNU Affero General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
      SPDX-License-Identifier: AGPL3.0-or-later
-*/
+ */
 /**
  * @file include/gnunet_pq_lib.h
  * @brief helper functions for Postgres DB interactions
@@ -47,15 +47,15 @@
  * @return -1 on error, number of offsets used in @a scratch otherwise
  */
 typedef int
-(*GNUNET_PQ_QueryConverter)(void *cls,
-			    const void *data,
-			    size_t data_len,
-			    void *param_values[],
-			    int param_lengths[],
-			    int param_formats[],
-			    unsigned int param_length,
-			    void *scratch[],
-			    unsigned int scratch_length);
+(*GNUNET_PQ_QueryConverter) (void *cls,
+                             const void *data,
+                             size_t data_len,
+                             void *param_values[],
+                             int param_lengths[],
+                             int param_formats[],
+                             unsigned int param_length,
+                             void *scratch[],
+                             unsigned int scratch_length);
 
 
 /**
@@ -63,7 +63,6 @@ typedef int
  */
 struct GNUNET_PQ_QueryParam
 {
-
   /**
    * Function for how to handle this type of entry.
    */
@@ -94,7 +93,10 @@ struct GNUNET_PQ_QueryParam
 /**
  * End of query parameter specification.
  */
-#define GNUNET_PQ_query_param_end { NULL, NULL, NULL, 0, 0 }
+#define GNUNET_PQ_query_param_end \
+  {                               \
+    NULL, NULL, NULL, 0, 0        \
+  }
 
 
 /**
@@ -105,9 +107,7 @@ struct GNUNET_PQ_QueryParam
  * @oaran ptr_size number of bytes in @a ptr
  */
 struct GNUNET_PQ_QueryParam
-GNUNET_PQ_query_param_fixed_size (const void *ptr,
-				  size_t ptr_size);
-
+GNUNET_PQ_query_param_fixed_size (const void *ptr, size_t ptr_size);
 
 
 /**
@@ -125,7 +125,8 @@ GNUNET_PQ_query_param_string (const char *ptr);
  *
  * @param x pointer to the query parameter to pass.
  */
-#define GNUNET_PQ_query_param_auto_from_type(x) GNUNET_PQ_query_param_fixed_size ((x), sizeof (*(x)))
+#define GNUNET_PQ_query_param_auto_from_type(x) \
+  GNUNET_PQ_query_param_fixed_size ((x), sizeof(*(x)))
 
 
 /**
@@ -135,7 +136,8 @@ GNUNET_PQ_query_param_string (const char *ptr);
  * @param x the query parameter to pass.
  */
 struct GNUNET_PQ_QueryParam
-GNUNET_PQ_query_param_rsa_public_key (const struct GNUNET_CRYPTO_RsaPublicKey *x);
+GNUNET_PQ_query_param_rsa_public_key (
+  const struct GNUNET_CRYPTO_RsaPublicKey *x);
 
 
 /**
@@ -145,7 +147,18 @@ GNUNET_PQ_query_param_rsa_public_key (const struct GNUNET_CRYPTO_RsaPublicKey *x
  * @param x the query parameter to pass
  */
 struct GNUNET_PQ_QueryParam
-GNUNET_PQ_query_param_rsa_signature (const struct GNUNET_CRYPTO_RsaSignature *x);
+GNUNET_PQ_query_param_rsa_signature (
+  const struct GNUNET_CRYPTO_RsaSignature *x);
+
+
+/**
+ * Generate query parameter for a relative time value.
+ * The database must store a 64-bit integer.
+ *
+ * @param x pointer to the query parameter to pass
+ */
+struct GNUNET_PQ_QueryParam
+GNUNET_PQ_query_param_relative_time (const struct GNUNET_TIME_Relative *x);
 
 
 /**
@@ -165,7 +178,8 @@ GNUNET_PQ_query_param_absolute_time (const struct GNUNET_TIME_Absolute *x);
  * @param x pointer to the query parameter to pass
  */
 struct GNUNET_PQ_QueryParam
-GNUNET_PQ_query_param_absolute_time_nbo (const struct GNUNET_TIME_AbsoluteNBO *x);
+GNUNET_PQ_query_param_absolute_time_nbo (
+  const struct GNUNET_TIME_AbsoluteNBO *x);
 
 
 /**
@@ -212,12 +226,12 @@ GNUNET_PQ_query_param_uint64 (const uint64_t *x);
  *   #GNUNET_SYSERR if a result was invalid (non-existing field or NULL)
  */
 typedef int
-(*GNUNET_PQ_ResultConverter)(void *cls,
-			     PGresult *result,
-			     int row,
-			     const char *fname,
-			     size_t *dst_size,
-			     void *dst);
+(*GNUNET_PQ_ResultConverter) (void *cls,
+                              PGresult *result,
+                              int row,
+                              const char *fname,
+                              size_t *dst_size,
+                              void *dst);
 
 
 /**
@@ -228,8 +242,8 @@ typedef int
  * @param rd result data to clean up
  */
 typedef void
-(*GNUNET_PQ_ResultCleanup)(void *cls,
-			   void *rd);
+(*GNUNET_PQ_ResultCleanup) (void *cls,
+                            void *rd);
 
 
 /**
@@ -237,7 +251,6 @@ typedef void
  */
 struct GNUNET_PQ_ResultSpec
 {
-
   /**
    * What is the format of the result?
    */
@@ -275,7 +288,6 @@ struct GNUNET_PQ_ResultSpec
    * Where to store actual size of the result.
    */
   size_t *result_size;
-
 };
 
 
@@ -284,7 +296,10 @@ struct GNUNET_PQ_ResultSpec
  *
  * @return array last entry for the result specification to use
  */
-#define GNUNET_PQ_result_spec_end { NULL, NULL, NULL, NULL, 0, NULL, NULL }
+#define GNUNET_PQ_result_spec_end         \
+  {                                       \
+    NULL, NULL, NULL, NULL, 0, NULL, NULL \
+  }
 
 
 /**
@@ -297,8 +312,8 @@ struct GNUNET_PQ_ResultSpec
  */
 struct GNUNET_PQ_ResultSpec
 GNUNET_PQ_result_spec_variable_size (const char *name,
-				     void **dst,
-				     size_t *sptr);
+                                     void **dst,
+                                     size_t *sptr);
 
 
 /**
@@ -310,10 +325,7 @@ GNUNET_PQ_result_spec_variable_size (const char *name,
  * @return array entry for the result specification to use
  */
 struct GNUNET_PQ_ResultSpec
-GNUNET_PQ_result_spec_fixed_size (const char *name,
-				  void *dst,
-				  size_t dst_size);
-
+GNUNET_PQ_result_spec_fixed_size (const char *name, void *dst, size_t dst_size);
 
 
 /**
@@ -323,7 +335,8 @@ GNUNET_PQ_result_spec_fixed_size (const char *name,
  * @param dst point to where to store the result, type fits expected result size
  * @return array entry for the result specification to use
  */
-#define GNUNET_PQ_result_spec_auto_from_type(name, dst) GNUNET_PQ_result_spec_fixed_size (name, (dst), sizeof (*(dst)))
+#define GNUNET_PQ_result_spec_auto_from_type(name, dst) \
+  GNUNET_PQ_result_spec_fixed_size (name, (dst), sizeof(*(dst)))
 
 
 /**
@@ -334,8 +347,7 @@ GNUNET_PQ_result_spec_fixed_size (const char *name,
  * @return array entry for the result specification to use
  */
 struct GNUNET_PQ_ResultSpec
-GNUNET_PQ_result_spec_string (const char *name,
-                              char **dst);
+GNUNET_PQ_result_spec_string (const char *name, char **dst);
 
 
 /**
@@ -347,7 +359,7 @@ GNUNET_PQ_result_spec_string (const char *name,
  */
 struct GNUNET_PQ_ResultSpec
 GNUNET_PQ_result_spec_rsa_public_key (const char *name,
-				      struct GNUNET_CRYPTO_RsaPublicKey **rsa);
+                                      struct GNUNET_CRYPTO_RsaPublicKey **rsa);
 
 
 /**
@@ -359,7 +371,7 @@ GNUNET_PQ_result_spec_rsa_public_key (const char *name,
  */
 struct GNUNET_PQ_ResultSpec
 GNUNET_PQ_result_spec_rsa_signature (const char *name,
-				     struct GNUNET_CRYPTO_RsaSignature **sig);
+                                     struct GNUNET_CRYPTO_RsaSignature **sig);
 
 
 /**
@@ -371,7 +383,19 @@ GNUNET_PQ_result_spec_rsa_signature (const char *name,
  */
 struct GNUNET_PQ_ResultSpec
 GNUNET_PQ_result_spec_absolute_time (const char *name,
-				     struct GNUNET_TIME_Absolute *at);
+                                     struct GNUNET_TIME_Absolute *at);
+
+
+/**
+ * Relative time expected.
+ *
+ * @param name name of the field in the table
+ * @param[out] rt where to store the result
+ * @return array entry for the result specification to use
+ */
+struct GNUNET_PQ_ResultSpec
+GNUNET_PQ_result_spec_relative_time (const char *name,
+                                     struct GNUNET_TIME_Relative *rt);
 
 
 /**
@@ -383,7 +407,7 @@ GNUNET_PQ_result_spec_absolute_time (const char *name,
  */
 struct GNUNET_PQ_ResultSpec
 GNUNET_PQ_result_spec_absolute_time_nbo (const char *name,
-					 struct GNUNET_TIME_AbsoluteNBO *at);
+                                         struct GNUNET_TIME_AbsoluteNBO *at);
 
 
 /**
@@ -394,8 +418,7 @@ GNUNET_PQ_result_spec_absolute_time_nbo (const char *name,
  * @return array entry for the result specification to use
  */
 struct GNUNET_PQ_ResultSpec
-GNUNET_PQ_result_spec_uint16 (const char *name,
-			      uint16_t *u16);
+GNUNET_PQ_result_spec_uint16 (const char *name, uint16_t *u16);
 
 
 /**
@@ -406,8 +429,7 @@ GNUNET_PQ_result_spec_uint16 (const char *name,
  * @return array entry for the result specification to use
  */
 struct GNUNET_PQ_ResultSpec
-GNUNET_PQ_result_spec_uint32 (const char *name,
-			      uint32_t *u32);
+GNUNET_PQ_result_spec_uint32 (const char *name, uint32_t *u32);
 
 
 /**
@@ -418,25 +440,30 @@ GNUNET_PQ_result_spec_uint32 (const char *name,
  * @return array entry for the result specification to use
  */
 struct GNUNET_PQ_ResultSpec
-GNUNET_PQ_result_spec_uint64 (const char *name,
-			      uint64_t *u64);
+GNUNET_PQ_result_spec_uint64 (const char *name, uint64_t *u64);
 
 
 /* ************************* pq.c functions ************************ */
 
+/**
+ * Postgres context.
+ */
+struct GNUNET_PQ_Context;
+
+
 /**
  * Execute a prepared statement.
  *
- * @param db_conn database connection
+ * @param db database context
  * @param name name of the prepared statement
  * @param params parameters to the statement
  * @return postgres result
  * @deprecated (should become an internal API)
  */
 PGresult *
-GNUNET_PQ_exec_prepared (PGconn *db_conn,
-			 const char *name,
-			 const struct GNUNET_PQ_QueryParam *params);
+GNUNET_PQ_exec_prepared (struct GNUNET_PQ_Context *db,
+                         const char *name,
+                         const struct GNUNET_PQ_QueryParam *params);
 
 
 /**
@@ -452,8 +479,8 @@ GNUNET_PQ_exec_prepared (PGconn *db_conn,
  */
 int
 GNUNET_PQ_extract_result (PGresult *result,
-			  struct GNUNET_PQ_ResultSpec *rs,
-			  int row);
+                          struct GNUNET_PQ_ResultSpec *rs,
+                          int row);
 
 
 /**
@@ -473,7 +500,7 @@ GNUNET_PQ_cleanup_result (struct GNUNET_PQ_ResultSpec *rs);
  * Check the @a result's error code to see what happened.
  * Also logs errors.
  *
- * @param connection connection to execute the statement in
+ * @param db database to execute the statement in
  * @param statement_name name of the statement that created @a result
  * @param result result to check
  * @return status code from the result, mapping PQ status
@@ -483,7 +510,7 @@ GNUNET_PQ_cleanup_result (struct GNUNET_PQ_ResultSpec *rs);
  * @deprecated (low level, let's see if we can do with just the high-level functions)
  */
 enum GNUNET_DB_QueryStatus
-GNUNET_PQ_eval_result (PGconn *connection,
+GNUNET_PQ_eval_result (struct GNUNET_PQ_Context *db,
                        const char *statement_name,
                        PGresult *result);
 
@@ -493,7 +520,7 @@ GNUNET_PQ_eval_result (PGconn *connection,
  * statement in @a connnection using the given @a params.  Returns the
  * resulting session state.
  *
- * @param connection connection to execute the statement in
+ * @param db database to execute the statement with
  * @param statement_name name of the statement
  * @param params parameters to give to the statement (#GNUNET_PQ_query_param_end-terminated)
  * @return status code from the result, mapping PQ status
@@ -505,7 +532,7 @@ GNUNET_PQ_eval_result (PGconn *connection,
  *         zero; if INSERT was successful, we return one.
  */
 enum GNUNET_DB_QueryStatus
-GNUNET_PQ_eval_prepared_non_select (PGconn *connection,
+GNUNET_PQ_eval_prepared_non_select (struct GNUNET_PQ_Context *db,
                                     const char *statement_name,
                                     const struct GNUNET_PQ_QueryParam *params);
 
@@ -519,9 +546,9 @@ GNUNET_PQ_eval_prepared_non_select (PGconn *connection,
  * @param num_result the number of results in @a result
  */
 typedef void
-(*GNUNET_PQ_PostgresResultHandler)(void *cls,
-                                   PGresult *result,
-                                   unsigned int num_results);
+(*GNUNET_PQ_PostgresResultHandler) (void *cls,
+                                    PGresult *result,
+                                    unsigned int num_results);
 
 
 /**
@@ -531,7 +558,7 @@ typedef void
  * status including the number of results given to @a rh (possibly zero).
  * @a rh will not have been called if the return value is negative.
  *
- * @param connection connection to execute the statement in
+ * @param db database to execute the statement with
  * @param statement_name name of the statement
  * @param params parameters to give to the statement (#GNUNET_PQ_query_param_end-terminated)
  * @param rh function to call with the result set, NULL to ignore
@@ -540,7 +567,7 @@ typedef void
  *         codes to `enum GNUNET_DB_QueryStatus`.
  */
 enum GNUNET_DB_QueryStatus
-GNUNET_PQ_eval_prepared_multi_select (PGconn *connection,
+GNUNET_PQ_eval_prepared_multi_select (struct GNUNET_PQ_Context *db,
                                       const char *statement_name,
                                       const struct GNUNET_PQ_QueryParam *params,
                                       GNUNET_PQ_PostgresResultHandler rh,
@@ -555,7 +582,7 @@ GNUNET_PQ_eval_prepared_multi_select (PGconn *connection,
  * value was #GNUNET_DB_STATUS_SUCCESS_ONE_RESULT.  Returns the
  * resulting session status.
  *
- * @param connection connection to execute the statement in
+ * @param db database to execute the statement with
  * @param statement_name name of the statement
  * @param params parameters to give to the statement (#GNUNET_PQ_query_param_end-terminated)
  * @param[in,out] rs result specification to use for storing the result of the query
@@ -563,9 +590,10 @@ GNUNET_PQ_eval_prepared_multi_select (PGconn *connection,
  *         codes to `enum GNUNET_DB_QueryStatus`.
  */
 enum GNUNET_DB_QueryStatus
-GNUNET_PQ_eval_prepared_singleton_select (PGconn *connection,
+GNUNET_PQ_eval_prepared_singleton_select (struct GNUNET_PQ_Context *db,
                                           const char *statement_name,
-                                          const struct GNUNET_PQ_QueryParam *params,
+                                          const struct
+                                          GNUNET_PQ_QueryParam *params,
                                           struct GNUNET_PQ_ResultSpec *rs);
 
 
@@ -576,8 +604,8 @@ GNUNET_PQ_eval_prepared_singleton_select (PGconn *connection,
  * Information needed to prepare a list of SQL statements using
  * #GNUNET_PQ_prepare_statements().
  */
-struct GNUNET_PQ_PreparedStatement {
-
+struct GNUNET_PQ_PreparedStatement
+{
   /**
    * Name of the statement.
    */
@@ -599,7 +627,10 @@ struct GNUNET_PQ_PreparedStatement {
 /**
  * Terminator for prepared statement list.
  */
-#define GNUNET_PQ_PREPARED_STATEMENT_END { NULL, NULL, 0 }
+#define GNUNET_PQ_PREPARED_STATEMENT_END \
+  {                                      \
+    NULL, NULL, 0                        \
+  }
 
 
 /**
@@ -619,14 +650,14 @@ GNUNET_PQ_make_prepare (const char *name,
 /**
  * Request creation of prepared statements @a ps from Postgres.
  *
- * @param connection connection to prepare the statements for
+ * @param db database to prepare the statements for
  * @param ps #GNUNET_PQ_PREPARED_STATEMENT_END-terminated array of prepared
  *            statements.
  * @return #GNUNET_OK on success,
  *         #GNUNET_SYSERR on error
  */
 int
-GNUNET_PQ_prepare_statements (PGconn *connection,
+GNUNET_PQ_prepare_statements (struct GNUNET_PQ_Context *db,
                               const struct GNUNET_PQ_PreparedStatement *ps);
 
 
@@ -637,8 +668,8 @@ GNUNET_PQ_prepare_statements (PGconn *connection,
  * Information needed to run a list of SQL statements using
  * #GNUNET_PQ_exec_statements().
  */
-struct GNUNET_PQ_ExecuteStatement {
-
+struct GNUNET_PQ_ExecuteStatement
+{
   /**
    * Actual SQL statement.
    */
@@ -648,14 +679,16 @@ struct GNUNET_PQ_ExecuteStatement {
    * Should we ignore errors?
    */
   int ignore_errors;
-
 };
 
 
 /**
  * Terminator for executable statement list.
  */
-#define GNUNET_PQ_EXECUTE_STATEMENT_END { NULL, GNUNET_SYSERR }
+#define GNUNET_PQ_EXECUTE_STATEMENT_END \
+  {                                     \
+    NULL, GNUNET_SYSERR                 \
+  }
 
 
 /**
@@ -682,14 +715,14 @@ GNUNET_PQ_make_try_execute (const char *sql);
 /**
  * Request execution of an array of statements @a es from Postgres.
  *
- * @param connection connection to execute the statements over
+ * @param pq database to execute the statements in
  * @param es #GNUNET_PQ_PREPARED_STATEMENT_END-terminated array of prepared
  *            statements.
  * @return #GNUNET_OK on success (modulo statements where errors can be ignored)
  *         #GNUNET_SYSERR on error
  */
 int
-GNUNET_PQ_exec_statements (PGconn *connection,
+GNUNET_PQ_exec_statements (struct GNUNET_PQ_Context *db,
                            const struct GNUNET_PQ_ExecuteStatement *es);
 
 
@@ -697,31 +730,101 @@ GNUNET_PQ_exec_statements (PGconn *connection,
 
 
 /**
- * Create a connection to the Postgres database using @a config_str
- * for the configuration.  Initialize logging via GNUnet's log
- * routines and disable Postgres's logger.
+ * Create a connection to the Postgres database using @a config_str for the
+ * configuration.  Initialize logging via GNUnet's log routines and disable
+ * Postgres's logger.  Also ensures that the statements in @a load_path and @a
+ * es are executed whenever we (re)connect to the database, and that the
+ * prepared statements in @a ps are "ready".  If statements in @es fail that
+ * were created with #GNUNET_PQ_make_execute(), then the entire operation
+ * fails.
+ *
+ * In @a load_path, a list of "$XXXX.sql" files is expected where $XXXX
+ * must be a sequence of contiguous integer values starting at 0000.
+ * These files are then loaded in sequence using "psql $config_str" before
+ * running statements from @e es.  The directory is inspected again on
+ * reconnect.
  *
  * @param config_str configuration to use
+ * @param load_path path to directory with SQL transactions to run, can be NULL
+ * @param es #GNUNET_PQ_PREPARED_STATEMENT_END-terminated
+ *            array of statements to execute upon EACH connection, can be NULL
+ * @param ps array of prepared statements to prepare, can be NULL
  * @return NULL on error
  */
-PGconn *
-GNUNET_PQ_connect (const char *config_str);
+struct GNUNET_PQ_Context *
+GNUNET_PQ_connect (const char *config_str,
+                   const char *load_path,
+                   const struct GNUNET_PQ_ExecuteStatement *es,
+                   const struct GNUNET_PQ_PreparedStatement *ps);
 
 
 /**
  * Connect to a postgres database using the configuration
- * option "CONFIG" in @a section.
+ * option "CONFIG" in @a section.  Also ensures that the
+ * statements in @a es are executed whenever we (re)connect to the
+ * database, and that the prepared statements in @a ps are "ready".
+ *
+ * The caller does not have to ensure that @a es and @a ps remain allocated
+ * and initialized in memory until #GNUNET_PQ_disconnect() is called, as a copy will be made.
  *
  * @param cfg configuration
  * @param section configuration section to use to get Postgres configuration options
+ * @param load_path_suffix suffix to append to the SQL_DIR in the configuration
+ * @param es #GNUNET_PQ_PREPARED_STATEMENT_END-terminated
+ *            array of statements to execute upon EACH connection, can be NULL
+ * @param ps array of prepared statements to prepare, can be NULL
  * @return the postgres handle, NULL on error
  */
-PGconn *
+struct GNUNET_PQ_Context *
 GNUNET_PQ_connect_with_cfg (const struct GNUNET_CONFIGURATION_Handle *cfg,
-                            const char *section);
+                            const char *section,
+                            const char *load_path_suffix,
+                            const struct GNUNET_PQ_ExecuteStatement *es,
+                            const struct GNUNET_PQ_PreparedStatement *ps);
 
 
+/**
+ * Reinitialize the database @a db if the connection is down.
+ *
+ * @param db database connection to reinitialize
+ */
+void
+GNUNET_PQ_reconnect_if_down (struct GNUNET_PQ_Context *db);
 
-#endif  /* GNUNET_PQ_LIB_H_ */
+
+/**
+ * Reinitialize the database @a db.
+ *
+ * @param db database connection to reinitialize
+ */
+void
+GNUNET_PQ_reconnect (struct GNUNET_PQ_Context *db);
+
+
+/**
+ * Within the @a db context, run all the SQL files
+ * from the @a load_path from 0000-9999.sql (as long
+ * as the files exist contiguously).
+ *
+ * @param db database context to use
+ * @param load_path where to find the XXXX.sql files
+ * @return #GNUNET_OK on success
+ */
+int
+GNUNET_PQ_run_sql (struct GNUNET_PQ_Context *db,
+                   const char *load_path);
+
+
+/**
+ * Disconnect from the database, destroying the prepared statements
+ * and releasing other associated resources.
+ *
+ * @param db database handle to disconnect (will be free'd)
+ */
+void
+GNUNET_PQ_disconnect (struct GNUNET_PQ_Context *db);
+
+
+#endif /* GNUNET_PQ_LIB_H_ */
 
 /* end of include/gnunet_pq_lib.h */
